function [x,xhist] = gradcon(A,b,x0,toll,maxit)
%GRADIENTE Implementa il metodo del gradiente coniugato
%  SINTASSI
%     [x,xhist] = gradcon(A,b,x0,toll,maxit)
%  INPUT
%     A     : Matrice simmetrica e definita positiva
%     b     : Vettore dei termini noti
%     x0    : Punto iniziale
%     toll  : Tolleranza del metodo
%     maxit : Numero massimo di iterazioni
%  OUTPUT
%     x     : ultimo punto trovato
%     xhist : matrice delle successioni

%   Autore : M. Venturin

x = x0(:);              % punto corrente
n = length(x);          % dimensione
iter = 1;               % iterazione corrente
xhist = zeros(n,maxit); % Allocazione vettore uscita
xhist(:,iter) = x;
r = b-A*x0;             % residuo iniziale
p = r;
bnrm2 = norm(b);        % norma iniziale
if bnrm2 < toll
   bnrm2 = 1.0;
end
errore = norm(r);       % errore iniziale

% ciclo principale
while errore > bnrm2*toll && iter<maxit
   iter = iter+1;       % iterazione
   w = A*p;             % w vett. ausiliario
   alpha = -r'*p/(p'*w);% alpha
   x = x-alpha*p;       % nuova x
   if mod(iter,50)==0
      r = b-A*x;        % nuovo residuo
   else
      r = r+alpha*w;    % nuovo residuo
   end   
   p = r-((p'*A*r)/(p'*w))*p; % p
   errore = norm(r);    % nuovo errore
   xhist(:,iter) = x;   % salvataggio x
end
xhist = xhist(:,1:iter);