function [x,xhist] = gradiente(A,b,x0,toll,maxit)
%GRADIENTE Implementa il metodo del gradiente
%  SINTASSI
%     [x,xhist] = gradiente(A,b,x0,toll,maxit)
%  INPUT
%     A     : Matrice simmetrica e definita positiva
%     b     : Vettore dei termini noti
%     x0    : Punto iniziale
%     toll  : Tolleranza del metodo
%     maxit : Numero massimo di iterazioni
%  OUTPUT
%     x     : ultimo punto trovato
%     xhist : matrice delle successioni

%   Autore : M. Venturin

x = x0(:);              % punto corrente
n = length(x);          % dimensione
iter = 1;               % iterazione corrente
xhist = zeros(n,maxit); % Allocazione vettore uscita
xhist(:,iter) = x;
r = b-A*x0;             % residuo iniziale
bnrm2 = norm(b);        % norma iniziale
if bnrm2 < toll
   bnrm2 = 1.0;
end
errore = norm(r);       % errore iniziale

% ciclo principale
while errore > bnrm2*toll && iter<maxit
   iter = iter+1;       % incr. iterazione
   z = A*r;             % z vett. ausiliario
   alpha = r'*r/(z'*r); % alpha
   x = x+alpha*r;       % x nuovo
   if mod(iter,50)==0
      r = b-A*x;        % nuovo residuo
   else
      r = r-alpha*z;    % nuovo residuo
   end
   errore = norm(r);    % nuovo errore
   xhist(:,iter) = x;   % salvataggio x
end
xhist = xhist(:,1:iter);