% FILE : pcgesempio.m

close all; clear all; clc;
%%% COSTRUZIONE DEL SISTEMA LINEARE
% Problema : - nabla(u) = 1 in un quadrato unitario
%                    u  = 0 sul bordo

% Generazione e visualizzazione della griglia
n = 24; G = numgrid('S',n);
figure; spy(G); title('Nodi della griglia');

% Discretizzazione operatore Laplaciano
% nei nodi interni
A = delsq(G);
figure; spy(A); title('Matrice sistema');

% Calcolo vettore dei termini noti nei nodi interni
N = sum(G(:)>0); % numero nodi interni
b = ones(N,1);   % vettore termini noti
% Sistema lineare da risolvere : A x = b (nodi interni)

%%% RISOLUZIONE CON METODO DIRETTO
% Risoluzione del sistema A x = b associato
% al problema di Laplace

% Chiamata al metodo diretto
u = A\b;

% Visualizzazione curve di livello soluzione
U = G; U(G>0) = full(u(G(G>0)));
figure; clabel(contourf(U)); prism; axis square ij

% Visualizzazione della soluzione
figure; mesh(U);
axis([0 n 0 n 0 max(max(U))]); axis square ij

% Residuo soluzione (6.9941e-013)
resd = norm(A*u-b);


% PCG CON PRECONDIZIONATORE DIAGONALE

% Calcolo del precondizionatore
D = spdiags(sqrt(spdiags(A,0)),0,size(A,1),size(A,2));

% Calcolo della soluzione
[upcg1,flag1,relres1,iter1,resvec1] = pcg(A,b,1e-8,50,D',D);

% Visualizzo andamento della soluzione
figure; h = semilogy(0:iter1,resvec1/norm(b),'-o');
set(h,'Linewidth',2);
xlabel('numero iterazione'); ylabel('residuo relativo');
grid on;


% PCG CON PRECONDIZIONATORE INCOMPLETO DI CHOLESKY

% Calcolo del precondizionatore
R = cholinc(A,1e-3);

% Calcolo della soluzione
[upcg2,flag2,relres2,iter2,resvec2] = pcg(A,b,1e-8,50,R',R);

% Visualizzo andamento della soluzione
figure; h = semilogy(0:iter2,resvec2/norm(b),'-o');
set(h,'Linewidth',2);
xlabel('numero iterazione'); ylabel('residuo relativo');
grid on;
% Pattern del precondizionatore
figure; spy(R'*R);

% ANALISI SPETTRALE

% Autovalori del sistema originario
eigo = eig(full(A));
% Visualizzazione del relativo spettro
figure; plot(real(eigo),imag(eigo),'xk');

% Autovalori del sistema precondizionato diag
eigp1 = eig(inv(full(D'))*full(A)*inv(full(D)));
% Visualizzazione del relativo spettro
figure; plot(real(eigp1),imag(eigp1),'xk');

% Autovalori del sistema precondizionato cholinc
eigp2 = eig(inv(full(R'))*full(A)*inv(full(R)));
% Visualizzazione del relativo spettro
figure; plot(real(eigp2),imag(eigp2),'xk');
